package com.blizzard.test;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

public class SimpleServletRequestImpl implements SimpleServletRequest {

	private String host, urlPath;
	private InetAddress clientAddress;
	private String[] headers;
	Map<String, String> urlParameters;
	
	public SimpleServletRequestImpl(){
		this.host = "";
		this.urlPath = "";
		this.urlParameters = new HashMap<String, String>();
		this.headers = null;
	}
	
	public SimpleServletRequestImpl(String[] headersFromParam){
		this.urlParameters = new HashMap<String, String>();
		this.setViaHeaders(headersFromParam);
	}
	
	public void setViaHeaders(String[] headersFromParam){
		this.headers = headersFromParam;
		try {
			this.clientAddress = InetAddress.getByName(this.getHeader("Connection request from: "));
		} catch (UnknownHostException e) {
			this.clientAddress = null;
		}	
		this.host = this.getHeader("Host: ");
		
		String pathAndParameters = this.getHeader("GET ");								//" HTTP/1.1".length() = 9
//System.out.println("SimpleServletRequestImpl.setViaHeaders - pathAndParameters: " + pathAndParameters);		
		pathAndParameters = pathAndParameters.substring(0, pathAndParameters.length() - 9); // replaces HTTP/x.x with nothing
		
		String[] pathAndParametersSorter = pathAndParameters.split("[\\?]");
		
		this.urlPath = pathAndParametersSorter[0];
		//System.out.println("SimpleServletRequestImpl - stored urlPath: " + this.urlPath);
		if(pathAndParametersSorter.length > 1){
			
			String[] listOfParameters;
			
			listOfParameters = pathAndParametersSorter[1].split("[&]");
			String[] temp;
			for(String paramSet: listOfParameters){
				temp = paramSet.split("[=]");
				
				if(temp.length > 1){
					this.urlParameters.put(temp[0], temp[1]);

				}else{
					this.urlParameters.put(temp[0], null);
				}

			}
		}
	}
	
	@Override
	public String getHost() {
		return host;
	}

	@Override
	public String getUrlPath() {
		return this.urlPath;
	}

	@Override
	public String getUrlParameter(String name) {
		if(this.urlParameters.containsKey(name)){
			return this.urlParameters.get(name);
		}else{
			return null;
		}
	}

	@Override
	public String getHeader(String name) {
		//String returnValue = null;
		for( String s: this.headers){
			if(s.startsWith(name)){
				return s.replace(name, "");
			}
		}
		return null;
	}

	@Override
	public InetAddress getClientAddress() {
		return this.clientAddress;
	}

}
