package com.blizzard.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SimpleServletThreadImpl implements SimpleServletThread, Runnable {
	private SimpleServletRequest request;
	private SimpleServletResponse response;
	
	public SimpleServletThreadImpl(SimpleServletRequest request,
			SimpleServletResponse response){
		this.request = request;
		this.response = response;
	}
	
	@Override
	public void doGet(SimpleServletRequest request,
			SimpleServletResponse response) {
		String relativeDirectoryToWebResources = "./resources";
		String resourceToRequest = relativeDirectoryToWebResources;

		List<String> defaultFileNameList; //add an ordered list for retrieving a directory resource. We want to serve up a default page instead.
		defaultFileNameList = new ArrayList<String>();
		defaultFileNameList.add("index.html");
		defaultFileNameList.add("index.htm");
		
		String output = null, header = null;
		List<Byte> content = null;
		
		
		File f = new File(relativeDirectoryToWebResources + request.getUrlPath());
		if( f.exists() ){
			resourceToRequest += request.getUrlPath();
			if(f.isDirectory()){
				//check for trailing /, if it doesn't exist, add it, since we need it.
				if(! request.getUrlPath().endsWith("/")){
					resourceToRequest += "/";
				}
				
				for(String indexFileName: defaultFileNameList){
					if(new File(resourceToRequest + indexFileName).exists()){
						resourceToRequest += indexFileName; 
						break;
					}
				}
				
				if((resourceToRequest.equals(request.getUrlPath())) || (resourceToRequest.equals(request.getUrlPath() + "/"))){
					// no index file found. we're looking only at a directory listing. NOPE!
					response.setStatusCode(NathansWebServer.HTTPSTATUSCODE_UNAUTHORIZED);
				}else{					
					content = this.getContentFromFile(resourceToRequest);
					response.setStatusCode(NathansWebServer.HTTPSTATUSCODE_OK);
				}
			}else if(f.isFile()){
				content = this.getContentFromFile(relativeDirectoryToWebResources + request.getUrlPath());
				response.setStatusCode(NathansWebServer.HTTPSTATUSCODE_OK);
				
			}else{ // !f.isFile() and !f.isDirectory()
				// not sure what else it could be! Not a file. Not a directory. It exists on the Filesystem.
				response.setStatusCode(NathansWebServer.HTTPSTATUSCODE_UNAUTHORIZED);
				// We don't know what it is, so we're going to deny access to it.
			}
		}else{ // !f.exists()  
			response.setStatusCode(NathansWebServer.HTTPSTATUSCODE_FILE_NOT_FOUND);
		}

		try {
			header = "HTTP/1.1 " + response.getStatusCode() + "\r\n";
			if((response.getMimeType() != null)&&(!response.getMimeType().equals(""))){
				header += "Content-Type: " + response.getMimeType() + "\r\n";
			}
			if(content != null){
				header += "Content-Length: " + content.size() + "\r\n";
			}
			output = "";
			output = header + "\r\n";
			//System.out.println(header);
			try{
				response.getOutputStream().write(output.getBytes());
				if(content != null){
					for(byte buffer: content){
						response.getOutputStream().write(buffer);
					}
				}
				
			}catch(SocketException e){
				//socket probably closed - nothing to do except finish out.
				System.out.println("Connection reset while sending data to client. Closing connection.");
				
			}finally{
				response.getOutputStream().close();
			}
			
		} catch (IOException e) {
			// an issue with response.getOutputStream() -- nothing we can do.
			// Just noted in comments, and Java will throw a fit if we don't catch it.
		}
	}

	@Override
	public void run() {
		this.doGet(this.request, this.response);
	}
	
	private List<Byte> getContentFromFile(String fileName){
		List<Byte> returnValue = new ArrayList<Byte>();
		String[] pathParts;
		String fileExtension = "";
		Map<String, String> fileTypeAssociations = new HashMap<String, String>();
		//extension -> mimeType
		fileTypeAssociations.put("jpg"	,	"image/jpeg"		);
		fileTypeAssociations.put("jpeg"	,	"image/jpeg"		);
		fileTypeAssociations.put("png"	,	"image/png"			);
		fileTypeAssociations.put("gif"	,	"image/gif"			);
		fileTypeAssociations.put("ico"	,	"image/x-icon"		);
		fileTypeAssociations.put("css"	,	"text/css"			);
		fileTypeAssociations.put("htm"	,	"text/html"			);
		fileTypeAssociations.put("html"	,	"text/html"			);
		fileTypeAssociations.put("js"	,	"text/javascript"	);
		fileTypeAssociations.put("xml"	,	"text/xml"			);
		
		// resourceToRequest should now be a full name, derived from a directory name + our index list.
		pathParts = fileName.split("[\\.]");
		if(pathParts.length > 1){
			fileExtension = pathParts[pathParts.length - 1];
			if(fileTypeAssociations.containsKey(fileExtension.toLowerCase())){
				response.setMimeType(fileTypeAssociations.get(fileExtension.toLowerCase()));
				response.setHeader("Content-Type", fileTypeAssociations.get(fileExtension.toLowerCase()));
			}else{
				//don't set header.
				response.setMimeType("");
			}
			try {
				InputStream input=new FileInputStream(fileName);
				int readData;
				while((readData=input.read())!=-1){
					returnValue.add((byte)readData);
				}
				
			} catch (FileNotFoundException e) {
				System.out.println("File " + fileName + " not found. " + e);
				//e.printStackTrace(); // should never happen, we already checked to see if it exists
			} catch (IOException e){
				System.out.println("IOException: " + e);
				//e.printStackTrace(); //
			}
		}
		return returnValue;
	}
}
